@@ -26,7 +26,9 @@
  */
 #include "tests/checkasm/checkasm.h"
 
+#include <errno.h>
 #include <math.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,13 +36,15 @@
 #include "src/cpu.h"
 
 #ifdef _WIN32
-#include <windows.h>
-#define COLOR_RED    FOREGROUND_RED
-#define COLOR_GREEN  FOREGROUND_GREEN
-#define COLOR_YELLOW (FOREGROUND_RED|FOREGROUND_GREEN)
+#ifndef SIGBUS
+/* non-standard, use the same value as mingw-w64 */
+#define SIGBUS 10
+#endif
+#ifndef ENABLE_VIRTUAL_TERMINAL_PROCESSING
+#define ENABLE_VIRTUAL_TERMINAL_PROCESSING 0x04
+#endif
 #else
 #include <unistd.h>
-#include <signal.h>
 #include <time.h>
 #include <pthread.h>
 #ifdef HAVE_PTHREAD_NP_H
@@ -49,17 +53,19 @@
 #ifdef __APPLE__
 #include <mach/mach_time.h>
 #endif
-#define COLOR_RED    1
-#define COLOR_GREEN  2
-#define COLOR_YELLOW 3
 #endif
 
+#define COLOR_RED    31
+#define COLOR_GREEN  32
+#define COLOR_YELLOW 33
+
 /* List of tests to invoke */
 static const struct {
     const char *name;
     void (*func)(void);
 } tests[] = {
     { "msac", checkasm_check_msac },
+    { "pal", checkasm_check_pal },
     { "refmvs", checkasm_check_refmvs },
 #if CONFIG_8BPC
     { "cdef_8bpc", checkasm_check_cdef_8bpc },
@@ -96,12 +102,29 @@ static const struct {
     { "AVX-512 (Ice Lake)", "avx512icl", DAV1D_X86_CPU_FLAG_AVX512ICL },
 #elif ARCH_AARCH64 || ARCH_ARM
     { "NEON",               "neon",      DAV1D_ARM_CPU_FLAG_NEON },
+    { "DOTPROD",            "dotprod",   DAV1D_ARM_CPU_FLAG_DOTPROD },
+    { "I8MM",               "i8mm",      DAV1D_ARM_CPU_FLAG_I8MM },
+#if ARCH_AARCH64
+    { "SVE",                "sve",       DAV1D_ARM_CPU_FLAG_SVE },
+    { "SVE2",               "sve2",      DAV1D_ARM_CPU_FLAG_SVE2 },
+#endif /* ARCH_AARCH64 */
+#elif ARCH_LOONGARCH
+    { "LSX",                "lsx",       DAV1D_LOONGARCH_CPU_FLAG_LSX },
+    { "LASX",               "lasx",      DAV1D_LOONGARCH_CPU_FLAG_LASX },
 #elif ARCH_PPC64LE
     { "VSX",                "vsx",       DAV1D_PPC_CPU_FLAG_VSX },
+#elif ARCH_RISCV
+    { "RVV",                "rvv",       DAV1D_RISCV_CPU_FLAG_V },
 #endif
     { 0 }
 };
 
+#if ARCH_AARCH64 && HAVE_SVE
+int checkasm_sve_length(void);
+#elif ARCH_RISCV
+int checkasm_get_vlenb(void);
+#endif
+
 typedef struct CheckasmFuncVersion {
     struct CheckasmFuncVersion *next;
     void *func;
@@ -119,6 +142,13 @@ typedef struct CheckasmFunc {
     char name[];
 } CheckasmFunc;
 
+typedef enum {
+    RUN_NORMAL = 0,
+    RUN_BENCHMARK,
+    RUN_CPUFLAG_LISTING,
+    RUN_FUNCTION_LISTING,
+} CheckasmRunMode;
+
 /* Internal state */
 static struct {
     CheckasmFunc *funcs;
@@ -133,11 +163,9 @@ static struct {
     const char *test_pattern;
     const char *function_pattern;
     unsigned seed;
-    int bench;
-    int bench_c;
+    CheckasmRunMode run_mode;
     int verbose;
-    int function_listing;
-    int catch_signals;
+    volatile sig_atomic_t catch_signals;
     int suffix_length;
     int max_function_name_length;
 #if ARCH_X86_64
@@ -241,48 +269,19 @@ int float_near_abs_eps_array_ulp(const float *const a, const float *const b,
 }
 
 /* Print colored text to stderr if the terminal supports it */
-static void color_printf(const int color, const char *const fmt, ...) {
-    static int8_t use_color = -1;
+static int use_printf_color;
+static void color_fprintf(FILE *const f, const int color, const char *const fmt, ...) {
     va_list arg;
 
-#ifdef _WIN32
-    static HANDLE con;
-    static WORD org_attributes;
-
-    if (use_color < 0) {
-        CONSOLE_SCREEN_BUFFER_INFO con_info;
-        con = GetStdHandle(STD_ERROR_HANDLE);
-        if (con && con != INVALID_HANDLE_VALUE &&
-            GetConsoleScreenBufferInfo(con, &con_info))
-        {
-            org_attributes = con_info.wAttributes;
-            use_color = 1;
-        } else
-            use_color = 0;
-    }
-    if (use_color)
-        SetConsoleTextAttribute(con, (org_attributes & 0xfff0) |
-                                (color & 0x0f));
-#else
-    if (use_color < 0) {
-        const char *const term = getenv("TERM");
-        use_color = term && strcmp(term, "dumb") && isatty(2);
-    }
-    if (use_color)
-        fprintf(stderr, "\x1b[%d;3%dm", (color & 0x08) >> 3, color & 0x07);
-#endif
+    if (use_printf_color)
+        fprintf(f, "\x1b[0;%dm", color);
 
     va_start(arg, fmt);
-    vfprintf(stderr, fmt, arg);
+    vfprintf(f, fmt, arg);
     va_end(arg);
 
-    if (use_color) {
-#ifdef _WIN32
-        SetConsoleTextAttribute(con, org_attributes);
-#else
-        fprintf(stderr, "\x1b[0m");
-#endif
-    }
+    if (use_printf_color)
+        fprintf(f, "\x1b[0m");
 }
 
 /* Deallocate a tree */
@@ -359,7 +358,7 @@ static void print_benchs(const CheckasmFunc *const f) {
 
         /* Only print functions with at least one assembly version */
         const CheckasmFuncVersion *v = &f->versions;
-        if ((state.bench_c || v->cpu || v->next) && v->iterations) {
+        if (v->iterations) {
             const double baseline = avg_cycles_per_call(v);
             do {
                 const int pad_length = 10 + state.max_function_name_length -
@@ -462,48 +461,51 @@ checkasm_context checkasm_context_buf;
 /* Crash handling: attempt to catch crashes and handle them
  * gracefully instead of just aborting abruptly. */
 #ifdef _WIN32
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
 static LONG NTAPI signal_handler(EXCEPTION_POINTERS *const e) {
     if (!state.catch_signals)
         return EXCEPTION_CONTINUE_SEARCH;
 
-    const char *err;
+    int s;
     switch (e->ExceptionRecord->ExceptionCode) {
     case EXCEPTION_FLT_DIVIDE_BY_ZERO:
     case EXCEPTION_INT_DIVIDE_BY_ZERO:
-        err = "fatal arithmetic error";
+        s = SIGFPE;
         break;
     case EXCEPTION_ILLEGAL_INSTRUCTION:
     case EXCEPTION_PRIV_INSTRUCTION:
-        err = "illegal instruction";
+        s = SIGILL;
         break;
     case EXCEPTION_ACCESS_VIOLATION:
     case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:
     case EXCEPTION_DATATYPE_MISALIGNMENT:
-    case EXCEPTION_IN_PAGE_ERROR:
     case EXCEPTION_STACK_OVERFLOW:
-        err = "segmentation fault";
+        s = SIGSEGV;
+        break;
+    case EXCEPTION_IN_PAGE_ERROR:
+        s = SIGBUS;
         break;
     default:
         return EXCEPTION_CONTINUE_SEARCH;
     }
     state.catch_signals = 0;
-    checkasm_fail_func(err);
-    checkasm_load_context();
+    checkasm_load_context(s);
     return EXCEPTION_CONTINUE_EXECUTION; /* never reached, but shuts up gcc */
 }
+#endif
 #else
+static void signal_handler(int s);
+
+static const struct sigaction signal_handler_act = {
+    .sa_handler = signal_handler,
+    .sa_flags = SA_RESETHAND,
+};
+
 static void signal_handler(const int s) {
     if (state.catch_signals) {
         state.catch_signals = 0;
-        checkasm_fail_func(s == SIGFPE ? "fatal arithmetic error" :
-                           s == SIGILL ? "illegal instruction" :
-                                         "segmentation fault");
-        checkasm_load_context();
-    } else {
-        /* fall back to the default signal handler */
-        static const struct sigaction default_sa = { .sa_handler = SIG_DFL };
-        sigaction(s, &default_sa, NULL);
-        raise(s);
+        sigaction(s, &signal_handler_act, NULL);
+        checkasm_load_context(s);
     }
 }
 #endif
@@ -548,7 +550,7 @@ static void check_cpu_flag(const char *const name, unsigned flag) {
 /* Print the name of the current CPU flag, but only do it once */
 static void print_cpu_name(void) {
     if (state.cpu_flag_name) {
-        color_printf(COLOR_YELLOW, "%s:\n", state.cpu_flag_name);
+        color_fprintf(stderr, COLOR_YELLOW, "%s:\n", state.cpu_flag_name);
         state.cpu_flag_name = NULL;
     }
 }
@@ -567,6 +569,13 @@ static unsigned get_seed(void) {
 #endif
 }
 
+static int checkasm_strtoul(unsigned long *const dst, const char *const str, const int base) {
+    char *end;
+    errno = 0;
+    *dst = strtoul(str, &end, base);
+    return errno || end == str || *end;
+}
+
 int main(int argc, char *argv[]) {
     state.seed = get_seed();
 
@@ -577,23 +586,21 @@ int main(int argc, char *argv[]) {
                     "    <random seed>              Numeric value to seed the rng\n"
                     "Options:\n"
                     "    --affinity=<cpu>           Run the process on CPU <cpu>\n"
-                    "    --test=<pattern>           Test only <pattern>\n"
+                    "    --test=<pattern> -t        Test only <pattern>\n"
                     "    --function=<pattern> -f    Test only the functions matching <pattern>\n"
                     "    --bench -b                 Benchmark the tested functions\n"
+                    "    --list-cpuflags            List available cpu flags\n"
                     "    --list-functions           List available functions\n"
                     "    --list-tests               List available tests\n"
-                    "    --bench-c -c               Benchmark the C-only functions\n"
                     "    --verbose -v               Print verbose output\n");
             return 0;
-        } else if (!strcmp(argv[1], "--bench-c") || !strcmp(argv[1], "-c")) {
-            state.bench_c = 1;
         } else if (!strcmp(argv[1], "--bench") || !strcmp(argv[1], "-b")) {
 #ifndef readtime
             fprintf(stderr,
                     "checkasm: --bench is not supported on your system\n");
             return 1;
 #endif
-            state.bench = 1;
+            state.run_mode = RUN_BENCHMARK;
         } else if (!strncmp(argv[1], "--test=", 7)) {
             state.test_pattern = argv[1] + 7;
         } else if (!strcmp(argv[1], "-t")) {
@@ -606,8 +613,11 @@ int main(int argc, char *argv[]) {
             state.function_pattern = argc > 1 ? argv[2] : "";
             argc--;
             argv++;
+        } else if (!strcmp(argv[1], "--list-cpuflags")) {
+            state.run_mode = RUN_CPUFLAG_LISTING;
+            break;
         } else if (!strcmp(argv[1], "--list-functions")) {
-            state.function_listing = 1;
+            state.run_mode = RUN_FUNCTION_LISTING;
         } else if (!strcmp(argv[1], "--list-tests")) {
             for (int i = 0; tests[i].name; i++)
                 printf("%s\n", tests[i].name);
@@ -615,15 +625,23 @@ int main(int argc, char *argv[]) {
         } else if (!strcmp(argv[1], "--verbose") || !strcmp(argv[1], "-v")) {
             state.verbose = 1;
         } else if (!strncmp(argv[1], "--affinity=", 11)) {
-            unsigned long affinity = strtoul(argv[1] + 11, NULL, 16);
+            const char *const s = argv[1] + 11;
+            unsigned long affinity;
+            if (checkasm_strtoul(&affinity, s, 16)) {
+                fprintf(stderr, "checkasm: invalid cpu affinity (%s)\n", s);
+                return 1;
+            }
 #ifdef _WIN32
+            int affinity_err;
+            HANDLE process = GetCurrentProcess();
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
             BOOL (WINAPI *spdcs)(HANDLE, const ULONG*, ULONG) =
                 (void*)GetProcAddress(GetModuleHandleW(L"kernel32.dll"), "SetProcessDefaultCpuSets");
-            HANDLE process = GetCurrentProcess();
-            int affinity_err;
-            if (spdcs) {
+            if (spdcs)
                 affinity_err = !spdcs(process, (ULONG[]){ affinity + 256 }, 1);
-            } else {
+            else
+#endif
+            {
                 if (affinity < sizeof(DWORD_PTR) * 8)
                     affinity_err = !SetProcessAffinityMask(process, (DWORD_PTR)1 << affinity);
                 else
@@ -652,7 +670,12 @@ int main(int argc, char *argv[]) {
             return 1;
 #endif
         } else {
-            state.seed = (unsigned) strtoul(argv[1], NULL, 10);
+            unsigned long seed;
+            if (checkasm_strtoul(&seed, argv[1], 10)) {
+                fprintf(stderr, "checkasm: unknown option (%s)\n", argv[1]);
+                return 1;
+            }
+            state.seed = (unsigned)seed;
         }
 
         argc--;
@@ -660,7 +683,7 @@ int main(int argc, char *argv[]) {
     }
 
 #if TRIM_DSP_FUNCTIONS
-    fprintf(stderr, "checkasm: reference functions unavailable\n");
+    fprintf(stderr, "checkasm: reference functions unavailable, reconfigure using '-Dtrim_dsp=false'\n");
     return 0;
 #endif
 
@@ -669,25 +692,30 @@ int main(int argc, char *argv[]) {
 #ifdef _WIN32
 #if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
     AddVectoredExceptionHandler(0, signal_handler);
+
+    HANDLE con = GetStdHandle(state.run_mode >= RUN_CPUFLAG_LISTING ?
+                              STD_OUTPUT_HANDLE : STD_ERROR_HANDLE);
+    DWORD con_mode = 0;
+    use_printf_color = con && con != INVALID_HANDLE_VALUE &&
+                       GetConsoleMode(con, &con_mode) &&
+                       SetConsoleMode(con, con_mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING);
 #endif
 #else
-    const struct sigaction sa = {
-        .sa_handler = signal_handler,
-        .sa_flags = SA_NODEFER,
-    };
-    sigaction(SIGBUS,  &sa, NULL);
-    sigaction(SIGFPE,  &sa, NULL);
-    sigaction(SIGILL,  &sa, NULL);
-    sigaction(SIGSEGV, &sa, NULL);
+    sigaction(SIGBUS,  &signal_handler_act, NULL);
+    sigaction(SIGFPE,  &signal_handler_act, NULL);
+    sigaction(SIGILL,  &signal_handler_act, NULL);
+    sigaction(SIGSEGV, &signal_handler_act, NULL);
+
+    if (isatty(state.run_mode >= RUN_CPUFLAG_LISTING ? 1 : 2)) {
+        const char *const term = getenv("TERM");
+        use_printf_color = term && strcmp(term, "dumb");
+    }
 #endif
 
 #ifdef readtime
-    if (state.bench) {
-        static int testing = 0;
-        checkasm_save_context();
-        if (!testing) {
+    if (state.run_mode == RUN_BENCHMARK) {
+        if (!checkasm_save_context()) {
             checkasm_set_signal_handler_state(1);
-            testing = 1;
             readtime();
             checkasm_set_signal_handler_state(0);
         } else {
@@ -699,11 +727,22 @@ int main(int argc, char *argv[]) {
 
     int ret = 0;
 
-    if (!state.function_listing) {
+    if (state.run_mode != RUN_FUNCTION_LISTING) {
+        const unsigned cpu_flags = dav1d_get_cpu_flags();
+        if (state.run_mode == RUN_CPUFLAG_LISTING) {
+            const int last_i = (int)(sizeof(cpus) / sizeof(*cpus)) - 2;
+            for (int i = 0; i <= last_i ; i++) {
+                if (cpus[i].flag & cpu_flags)
+                    color_fprintf(stdout, COLOR_GREEN, "%s", cpus[i].suffix);
+                else
+                    color_fprintf(stdout, COLOR_RED, "~%s", cpus[i].suffix);
+                printf(i == last_i ? "\n" : ", ");
+            }
+            return 0;
+        }
 #if ARCH_X86_64
         void checkasm_warmup_avx2(void);
         void checkasm_warmup_avx512(void);
-        const unsigned cpu_flags = dav1d_get_cpu_flags();
         if (cpu_flags & DAV1D_X86_CPU_FLAG_AVX512ICL)
             state.simd_warmup = checkasm_warmup_avx512;
         else if (cpu_flags & DAV1D_X86_CPU_FLAG_AVX2)
@@ -717,6 +756,18 @@ int main(int argc, char *argv[]) {
         for (size_t len = strlen(name); len && name[len-1] == ' '; len--)
             name[len-1] = '\0'; /* trim trailing whitespace */
         fprintf(stderr, "checkasm: %s (%08X) using random seed %u\n", name, cpuid, state.seed);
+#elif ARCH_RISCV
+        char buf[32] = "";
+        if (cpu_flags & DAV1D_RISCV_CPU_FLAG_V) {
+            const int vlen = 8*checkasm_get_vlenb();
+            snprintf(buf, sizeof(buf), "VLEN=%i bits, ", vlen);
+        }
+        fprintf(stderr, "checkasm: %susing random seed %u\n", buf, state.seed);
+#elif ARCH_AARCH64 && HAVE_SVE
+        char buf[48] = "";
+        if (cpu_flags & DAV1D_ARM_CPU_FLAG_SVE)
+            snprintf(buf, sizeof(buf), "SVE %d bits, ", checkasm_sve_length());
+        fprintf(stderr, "checkasm: %susing random seed %u\n", buf, state.seed);
 #else
         fprintf(stderr, "checkasm: using random seed %u\n", state.seed);
 #endif
@@ -726,7 +777,7 @@ int main(int argc, char *argv[]) {
     for (int i = 0; cpus[i].flag; i++)
         check_cpu_flag(cpus[i].name, cpus[i].flag);
 
-    if (state.function_listing) {
+    if (state.run_mode == RUN_FUNCTION_LISTING) {
         print_functions(state.funcs);
     } else if (state.num_failed) {
         fprintf(stderr, "checkasm: %d of %d tests failed\n",
@@ -738,7 +789,7 @@ int main(int argc, char *argv[]) {
         else
             fprintf(stderr, "checkasm: no tests to perform\n");
 #ifdef readtime
-        if (state.bench && state.max_function_name_length) {
+        if (state.run_mode == RUN_BENCHMARK && state.max_function_name_length) {
             state.nop_time = measure_nop_time();
             if (state.verbose)
                 printf("nop:%*.1f\n", state.max_function_name_length + 6, state.nop_time);
@@ -798,7 +849,7 @@ void *checkasm_check_func(void *const func, const char *const name, ...) {
     v->ok = 1;
     v->cpu = state.cpu_flag;
     state.current_func_ver = v;
-    if (state.function_listing) /* Save function names without running tests */
+    if (state.run_mode == RUN_FUNCTION_LISTING) /* Save function names without running tests */
         return NULL;
 
     xor128_srand(state.seed);
@@ -811,7 +862,7 @@ void *checkasm_check_func(void *const func, const char *const name, ...) {
 
 /* Decide whether or not the current function needs to be benchmarked */
 int checkasm_bench_func(void) {
-    return !state.num_failed && state.bench;
+    return !state.num_failed && state.run_mode == RUN_BENCHMARK;
 }
 
 /* Indicate that the current test has failed, return whether verbose printing
@@ -860,9 +911,9 @@ void checkasm_report(const char *const name, ...) {
         fprintf(stderr, "%*c", imax(pad_length, 0) + 2, '[');
 
         if (state.num_failed == prev_failed)
-            color_printf(COLOR_GREEN, "OK");
+            color_fprintf(stderr, COLOR_GREEN, "OK");
         else
-            color_printf(COLOR_RED, "FAILED");
+            color_fprintf(stderr, COLOR_RED, "FAILED");
         fprintf(stderr, "]\n");
 
         prev_checked = state.num_checked;
@@ -886,6 +937,16 @@ void checkasm_set_signal_handler_state(const int enabled) {
     state.catch_signals = enabled;
 }
 
+int checkasm_handle_signal(const int s) {
+    if (s) {
+        checkasm_fail_func(s == SIGFPE ? "fatal arithmetic error" :
+                           s == SIGILL ? "illegal instruction" :
+                           s == SIGBUS ? "bus error" :
+                                         "segmentation fault");
+    }
+    return s;
+}
+
 static int check_err(const char *const file, const int line,
                      const char *const name, const int w, const int h,
                      int *const err)
